
import re
from mqttlogic import Operator

class Expression(object):

    def __init__(self, expression, hysteresis=0):
        self.parseExpression(expression, hysteresis)

    def parseExpression(self, expression, hysteresis):
        self.hysteresis = 0
        self.hysteresisOperator = None
        self.hysteresisValue = None

        numericRegex = '(sensors/(air|reed)/(\d+)/reading):([A-Za-z]+) (>|=|<|>=|<=) (\d+\.?\d*)'
        m = re.fullmatch(numericRegex, expression)
        if m != None:
            self.mqttPath = m.group(1)
            #sensorId = int(m.group(3))
            self.attribute = m.group(4) # s.g. like hdt (humidity) or tmp (temperature) or rs (reedstatus)
            self.operator = Operator.getOperator(m.group(5))
            self.value = float(m.group(6))
            self.hysteresis = hysteresis
        else:
            stringRegex = '(sensors/(air|reed)/(\d+)/reading):([A-Za-z]+) (!?=) ([A-Za-z]+)'
            m = re.fullmatch(stringRegex, expression)
            if m != None:
                self.mqttPath = m.group(1)
                #sensorId = int(m.group(3))
                self.attribute = m.group(4)
                self.operator = Operator.getOperator(m.group(5))
                self.value = m.group(6)


    #return None if alarm should not trigger at all, return the delay when alarm should trigger otherwise
    def evaluate(self, mqttPath, jsonObj):

        if mqttPath != self.mqttPath:
            return False

        #if isinstance(1, str):
        currentMeasurement = jsonObj.get(self.attribute, None)

        expressionMatch = self.hysteresisOperator is None and self.operator.evaluate(currentMeasurement, self.value)


        #create a new expression that informs us when hysteresis level is reached
        #e.g. if this expression is: temperature > 25.5 create an expression with temperature < 25.5-hysteresis
        if self.hysteresisOperator is None and expressionMatch and self.hysteresis > 0 and (self.operator.isLessComparison() or self.operator.isGreaterComparison()):
            reverseOp = self.operator.getReverseOperator()
            if self.operator.isLessComparison():
                reverseValue = self.value + self.hysteresis
            else:
                reverseValue = self.value - self.hysteresis

            self.hysteresisOperator = reverseOp
            self.hysteresisValue = reverseValue
            print ("Setting hysteresis value to " + str(self.hysteresisValue))
        elif self.hysteresisOperator and self.hysteresisOperator.evaluate(currentMeasurement, self.hysteresisValue):
            #as soon as the hysteresis expression matches our original expression should be set active again
            self.hysteresisOperator = None
            self.hysteresisValue = None
            print ("Clearing hysteresis")

        return expressionMatch
